@@ -1,3 +1,5 @@
+# encoding: utf-8
+
 # The MIT License
 #
 # Copyright (c) 2013 Sebastian Ramacher
@@ -20,8 +22,11 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 # THE SOFTWARE.
 
+from __future__ import absolute_import
+
 import sys
 
-if __name__ == '__main__':
-    from bpdb import main
+if __name__ == "__main__":
+    from . import main
+
     sys.exit(main())
