@@ -8,15 +8,17 @@ import curtsies.events
 
 class ReloadEvent(curtsies.events.Event):
     """Request to rerun REPL session ASAP because imported modules changed"""
-    def __init__(self, files_modified=('?',)):
+
+    def __init__(self, files_modified=("?",)):
         self.files_modified = files_modified
 
     def __repr__(self):
-        return "<ReloadEvent from %s>" % (' & '.join(self.files_modified))
+        return "<ReloadEvent from %s>" % (" & ".join(self.files_modified))
 
 
 class RefreshRequestEvent(curtsies.events.Event):
     """Request to refresh REPL display ASAP"""
+
     def __repr__(self):
         return "<RefreshRequestEvent for now>"
 
@@ -26,12 +28,14 @@ class ScheduledRefreshRequestEvent(curtsies.events.ScheduledEvent):
 
     Used to schedule the disappearance of status bar message that only shows
     for a few seconds"""
+
     def __init__(self, when):
-        self.when = when  # time.time() + how long
+        super(ScheduledRefreshRequestEvent, self).__init__(when)
 
     def __repr__(self):
-        return ("<RefreshRequestEvent for %s seconds from now>" %
-                (self.when - time.time()))
+        return "<RefreshRequestEvent for %s seconds from now>" % (
+            self.when - time.time()
+        )
 
 
 class RunStartupFileEvent(curtsies.events.Event):
@@ -40,5 +44,6 @@ class RunStartupFileEvent(curtsies.events.Event):
 
 class UndoEvent(curtsies.events.Event):
     """Request to undo."""
+
     def __init__(self, n=1):
         self.n = n
