@@ -19,6 +19,8 @@ from pygments import format as pygformat
 from bpython._py3compat import PythonLexer
 from pygments.formatters import TerminalFormatter
 
+from wcwidth import wcswidth
+
 import blessings
 
 import curtsies
@@ -28,6 +30,7 @@ from curtsies import events
 
 import bpython
 from bpython.repl import Repl as BpythonRepl, SourceNotFound
+from bpython.repl import LineTypeTranslator as LineType
 from bpython.config import (
     Struct,
     loadini,
@@ -220,7 +223,7 @@ class FakeStdin(object):
 
     @property
     def encoding(self):
-        return "UTF8"
+        return sys.__stdin__.encoding
 
     # TODO write a read() method?
 
@@ -274,6 +277,31 @@ class ImportFinder(object):
         self.watcher = watcher
         self.old_meta_path = old_meta_path
 
+    def find_distributions(self, context):
+        for finder in self.old_meta_path:
+            distribution_finder = getattr(finder, "find_distributions", None)
+            if distribution_finder is not None:
+                loader = finder.find_distributions(context)
+                if loader is not None:
+                    return loader
+
+        return None
+
+    def find_spec(self, fullname, path, target=None):
+        for finder in self.old_meta_path:
+            # Consider the finder only if it implements find_spec
+            if getattr(finder, "find_spec", None) is None:
+                continue
+            # Attempt to find the spec
+            spec = finder.find_spec(fullname, path, target)
+            if spec is not None:
+                if getattr(spec, "__loader__", None) is not None:
+                    # Patch the loader to enable reloading
+                    spec.__loader__ = ImportLoader(
+                        self.watcher, spec.__loader__
+                    )
+                return spec
+
     def find_module(self, fullname, path=None):
         for finder in self.old_meta_path:
             loader = finder.find_module(fullname, path)
@@ -377,20 +405,28 @@ class BaseRepl(BpythonRepl):
         # so we're just using the same object
         self.interact = self.status_bar
 
-        # line currently being edited, without ps1 (usually '>>> ')
+        # logical line currently being edited, without ps1 (usually '>>> ')
         self._current_line = ""
 
         # current line of output - stdout and stdin go here
         self.current_stdouterr_line = ""
 
-        # lines separated whenever logical line
-        # length goes over what the terminal width
-        # was at the time of original output
+        # this is every line that's been displayed (input and output)
+        # as with formatting applied. Logical lines that exceeded the terminal width
+        # at the time of output are split across multiple entries in this list.
         self.display_lines = []
 
         # this is every line that's been executed; it gets smaller on rewind
         self.history = []
 
+        # This is every logical line that's been displayed, both input and output.
+        # Like self.history, lines are unwrapped, uncolored, and without prompt.
+        # Entries are tuples, where
+        #   - the first element the line (string, not fmtsr)
+        #   - the second element is one of 2 global constants: "input" or "output"
+        #     (use LineType.INPUT or LineType.OUTPUT to avoid typing these strings)
+        self.all_logical_lines = []
+
         # formatted version of lines in the buffer kept around so we can
         # unhighlight parens using self.reprint_line as called by bpython.Repl
         self.display_buffer = []
@@ -399,7 +435,7 @@ class BaseRepl(BpythonRepl):
         # because there wasn't room to display everything
         self.scroll_offset = 0
 
-        # from the left, 0 means first char
+        # cursor position relative to start of current_line, 0 is first char
         self._cursor_offset = 0
 
         self.orig_tcattrs = orig_tcattrs
@@ -411,12 +447,12 @@ class BaseRepl(BpythonRepl):
         self.stdout = FakeOutput(
             self.coderunner,
             self.send_to_stdouterr,
-            fileno=sys.__stdout__.fileno(),
+            real_fileobj=sys.__stdout__,
         )
         self.stderr = FakeOutput(
             self.coderunner,
             self.send_to_stdouterr,
-            fileno=sys.__stderr__.fileno(),
+            real_fileobj=sys.__stderr__,
         )
         self.stdin = FakeStdin(self.coderunner, self, self.edit_keys)
 
@@ -711,8 +747,9 @@ class BaseRepl(BpythonRepl):
             )
             and self.config.curtsies_right_arrow_completion
             and self.cursor_offset == len(self.current_line)
+            # if at end of current line and user presses RIGHT (to autocomplete)
         ):
-
+            # then autocomplete
             self.current_line += self.current_suggestion
             self.cursor_offset = len(self.current_line)
         elif e in ("<UP>",) + key_dispatch[self.config.up_one_line_key]:
@@ -765,6 +802,8 @@ class BaseRepl(BpythonRepl):
             self.on_tab(back=True)
         elif e in key_dispatch[self.config.undo_key]:  # ctrl-r for undo
             self.prompt_undo()
+        elif e in key_dispatch[self.config.redo_key]:  # ctrl-g for redo
+            self.redo()
         elif e in key_dispatch[self.config.save_key]:  # ctrl-s for save
             greenlet.greenlet(self.write2file).switch()
         elif e in key_dispatch[self.config.pastebin_key]:  # F8 for pastebin
@@ -848,14 +887,18 @@ class BaseRepl(BpythonRepl):
         else:
             self.cut_buffer = cut
 
-    def on_enter(self, insert_into_history=True, reset_rl_history=True):
+    def on_enter(self, new_code=True, reset_rl_history=True):
         # so the cursor isn't touching a paren TODO: necessary?
+        if new_code:
+            self.redo_stack = []
+
         self._set_cursor_offset(-1, update_completion=False)
         if reset_rl_history:
             self.rl_history.reset()
 
         self.history.append(self.current_line)
-        self.push(self.current_line, insert_into_history=insert_into_history)
+        self.all_logical_lines.append((self.current_line, LineType.INPUT))
+        self.push(self.current_line, insert_into_history=new_code)
 
     def on_tab(self, back=False):
         """Do something on tab key
@@ -965,6 +1008,9 @@ class BaseRepl(BpythonRepl):
             self.add_normal_character(e)
 
     def send_current_block_to_external_editor(self, filename=None):
+        """"
+        Sends the current code block to external editor to be edited. Usually bound to C-x.
+        """
         text = self.send_to_external_editor(self.get_current_block())
         lines = [line for line in text.split("\n")]
         while lines and not lines[-1].split():
@@ -977,15 +1023,12 @@ class BaseRepl(BpythonRepl):
         self.cursor_offset = len(self.current_line)
 
     def send_session_to_external_editor(self, filename=None):
+        """
+        Sends entire bpython session to external editor to be edited. Usually bound to F7.         
+        """
         for_editor = EDIT_SESSION_HEADER
-        for_editor += "\n".join(
-            line[len(self.ps1) :]
-            if line.startswith(self.ps1)
-            else line[len(self.ps2) :]
-            if line.startswith(self.ps2)
-            else "### " + line
-            for line in self.getstdout().split("\n")
-        )
+        for_editor += self.get_session_formatted_for_file()
+
         text = self.send_to_external_editor(for_editor)
         if text == for_editor:
             self.status_bar.message(
@@ -999,7 +1042,9 @@ class BaseRepl(BpythonRepl):
             current_line = lines[-1][4:]
         else:
             current_line = ""
-        from_editor = [line for line in lines if line[:3] != "###"]
+        from_editor = [
+            line for line in lines if line[:6] != "# OUT:" and line[:3] != "###"
+        ]
         if all(not line.strip() for line in from_editor):
             self.status_bar.message(
                 _("Session not reevaluated because saved file was blank")
@@ -1009,7 +1054,7 @@ class BaseRepl(BpythonRepl):
         source = preprocess("\n".join(from_editor), self.interp.compile)
         lines = source.split("\n")
         self.history = lines
-        self.reevaluate(insert_into_history=True)
+        self.reevaluate(new_code=True)
         self.current_line = current_line
         self.cursor_offset = len(self.current_line)
         self.status_bar.message(_("Session edited and reevaluated"))
@@ -1020,7 +1065,7 @@ class BaseRepl(BpythonRepl):
         cursor, line = self.cursor_offset, self.current_line
         for modname in set(sys.modules.keys()) - self.original_modules:
             del sys.modules[modname]
-        self.reevaluate(insert_into_history=True)
+        self.reevaluate(new_code=False)
         self.cursor_offset, self.current_line = cursor, line
         self.status_bar.message(
             _("Reloaded at %s by user.") % (time.strftime("%X"),)
@@ -1119,6 +1164,8 @@ class BaseRepl(BpythonRepl):
 
         If the interpreter successfully runs the code, clear the buffer
         """
+        # Note that push() overrides its parent without calling it, unlike
+        # urwid and cli which implement custom behavior and call repl.Repl.push
         if self.paste_mode:
             self.saved_indent = 0
         else:
@@ -1220,6 +1267,7 @@ class BaseRepl(BpythonRepl):
         if remove_from_history:
             for unused in self.buffer:
                 self.history.pop()
+                self.all_logical_lines.pop()
         self.buffer = []
         self.cursor_offset = 0
         self.saved_indent = 0
@@ -1227,6 +1275,9 @@ class BaseRepl(BpythonRepl):
         self.cursor_offset = len(self.current_line)
 
     def get_current_block(self):
+        """
+        Returns the current code block as string (without prompts)
+        """
         return "\n".join(self.buffer + [self.current_line])
 
     def send_to_stdouterr(self, output):
@@ -1254,6 +1305,13 @@ class BaseRepl(BpythonRepl):
                     [],
                 )
             )
+            # These can be FmtStrs, but self.all_logical_lines only wants strings
+            for line in [self.current_stdouterr_line] + lines[1:-1]:
+                if isinstance(line, FmtStr):
+                    self.all_logical_lines.append((line.s, LineType.OUTPUT))
+                else:
+                    self.all_logical_lines.append((line, LineType.OUTPUT))
+
             self.current_stdouterr_line = lines[-1]
         logger.debug("display_lines: %r", self.display_lines)
 
@@ -1344,7 +1402,11 @@ class BaseRepl(BpythonRepl):
 
     @property
     def current_cursor_line_without_suggestion(self):
-        """Current line, either output/input or Python prompt + code"""
+        """
+        Current line, either output/input or Python prompt + code
+
+        :returns: FmtStr
+        """
         value = self.current_output_line + (
             "" if self.coderunner.running else self.display_line_with_prompt
         )
@@ -1381,6 +1443,23 @@ class BaseRepl(BpythonRepl):
         self.current_stdouterr_line = ""
         self.stdin.current_line = "\n"
 
+    def number_of_padding_chars_on_current_cursor_line(self):
+        """ To avoid cutting off two-column characters at the end of lines where
+        there's only one column left, curtsies adds a padding char (u' ').
+        It's important to know about these for cursor positioning.
+
+        Should return zero unless there are fullwidth characters. """
+        full_line = self.current_cursor_line_without_suggestion
+        line_with_padding = "".join(
+            line.s
+            for line in paint.display_linize(
+                self.current_cursor_line_without_suggestion.s, self.width
+            )
+        )
+
+        # the difference in length here is how much padding there is
+        return len(line_with_padding) - len(full_line)
+
     def paint(
         self,
         about_to_exit=False,
@@ -1541,7 +1620,8 @@ class BaseRepl(BpythonRepl):
 
         if self.stdin.has_focus:
             cursor_row, cursor_column = divmod(
-                len(self.current_stdouterr_line) + self.stdin.cursor_offset,
+                wcswidth(self.current_stdouterr_line)
+                + wcswidth(self.stdin.current_line[: self.stdin.cursor_offset]),
                 width,
             )
             assert cursor_column >= 0, cursor_column
@@ -1559,13 +1639,14 @@ class BaseRepl(BpythonRepl):
                 len(self.current_line),
                 self.cursor_offset,
             )
-        else:
+        else:  # Common case for determining cursor position
             cursor_row, cursor_column = divmod(
                 (
-                    len(self.current_cursor_line_without_suggestion)
-                    - len(self.current_line)
-                    + self.cursor_offset
-                ),
+                    wcswidth(self.current_cursor_line_without_suggestion.s)
+                    - wcswidth(self.current_line)
+                    + wcswidth(self.current_line[: self.cursor_offset])
+                )
+                + self.number_of_padding_chars_on_current_cursor_line(),
                 width,
             )
             assert cursor_column >= 0, (
@@ -1782,11 +1863,13 @@ class BaseRepl(BpythonRepl):
         self.display_buffer.pop()
         self.buffer.pop()
         self.history.pop()
+        self.all_logical_lines.pop()
 
     def take_back_empty_line(self):
         assert self.history and not self.history[-1]
         self.history.pop()
         self.display_lines.pop()
+        self.all_logical_lines.pop()
 
     def prompt_undo(self):
         if self.buffer:
@@ -1801,7 +1884,16 @@ class BaseRepl(BpythonRepl):
 
         greenlet.greenlet(prompt_for_undo).switch()
 
-    def reevaluate(self, insert_into_history=False):
+    def redo(self):
+        if self.redo_stack:
+            temp = self.redo_stack.pop()
+            self.history.append(temp)
+            self.all_logical_lines.append((temp, LineType.INPUT))
+            self.push(temp)
+        else:
+            self.status_bar.message("Nothing to redo.")
+
+    def reevaluate(self, new_code=False):
         """bpython.Repl.undo calls this"""
         if self.watcher:
             self.watcher.reset()
@@ -1809,6 +1901,7 @@ class BaseRepl(BpythonRepl):
         old_display_lines = self.display_lines
         self.history = []
         self.display_lines = []
+        self.all_logical_lines = []
 
         if not self.weak_rewind:
             self.interp = self.interp.__class__()
@@ -1825,7 +1918,7 @@ class BaseRepl(BpythonRepl):
         sys.stdin = ReevaluateFakeStdin(self.stdin, self)
         for line in old_logical_lines:
             self._current_line = line
-            self.on_enter(insert_into_history=insert_into_history)
+            self.on_enter(new_code=new_code)
             while self.fake_refresh_requested:
                 self.fake_refresh_requested = False
                 self.process_event(bpythonevents.RefreshRequestEvent())
@@ -1865,14 +1958,17 @@ class BaseRepl(BpythonRepl):
     def initialize_interp(self):
         self.coderunner.interp.locals["_repl"] = self
         self.coderunner.interp.runsource(
-            "from bpython.curtsiesfrontend._internal " "import _Helper"
+            "from bpython.curtsiesfrontend._internal import _Helper\n"
         )
         self.coderunner.interp.runsource("help = _Helper(_repl)\n")
+        self.coderunner.interp.runsource("del _Helper\n")
 
         del self.coderunner.interp.locals["_repl"]
-        del self.coderunner.interp.locals["_Helper"]
 
     def getstdout(self):
+        """
+        Returns a string of the current bpython session, wrapped, WITH prompts.
+        """
         lines = self.lines_for_display + [self.current_line_formatted]
         s = (
             "\n".join(x.s if isinstance(x, FmtStr) else x for x in lines)
