@@ -26,17 +26,20 @@ def display_linize(msg, columns, blank_line=False):
     """Returns lines obtained by splitting msg over multiple lines.
 
     Warning: if msg is empty, returns an empty list of lines"""
-    display_lines = (
-        [
+    if not msg:
+        return [""] if blank_line else []
+    msg = fmtstr(msg)
+    try:
+        display_lines = list(msg.width_aware_splitlines(columns))
+    # use old method if wcwidth can't determine width of msg
+    except ValueError:
+        display_lines = [
             msg[start:end]
             for start, end in zip(
                 range(0, len(msg), columns),
                 range(columns, len(msg) + columns, columns),
             )
         ]
-        if msg
-        else ([""] if blank_line else [])
-    )
     return display_lines
 
 
