@@ -388,6 +388,16 @@ class SourceNotFound(Exception):
     """Exception raised when the requested source could not be found."""
 
 
+class LineTypeTranslator(object):
+    """ Used when adding a tuple to all_logical_lines, to get input / output values
+    having to actually type/know the strings """
+
+    # TODO use Enum once we drop support for Python 2
+
+    INPUT = "input"
+    OUTPUT = "output"
+
+
 class Repl(object):
     """Implements the necessary guff for a Python-repl-alike interface
 
@@ -438,8 +448,11 @@ class Repl(object):
         self.rl_history = History(
             duplicates=config.hist_duplicates, hist_size=config.hist_length
         )
-        self.s_hist = []
-        self.history = []
+        # all input and output, stored as old style format strings
+        # (\x01, \x02, ...) for cli.py
+        self.screen_hist = []
+        self.history = []  # commands executed since beginning of session
+        self.redo_stack = []
         self.evaluating = False
         self.matches_iter = MatchesIterator()
         self.funcprops = None
@@ -814,21 +827,36 @@ class Repl(object):
             indentation = 0
         return indentation
 
-    def formatforfile(self, session_ouput):
+    def get_session_formatted_for_file(self):
         """Format the stdout buffer to something suitable for writing to disk,
         i.e. without >>> and ... at input lines and with "# OUT: " prepended to
-        output lines."""
+        output lines and "### " prepended to current line"""
 
-        def process():
-            for line in session_ouput.split("\n"):
-                if line.startswith(self.ps1):
-                    yield line[len(self.ps1) :]
-                elif line.startswith(self.ps2):
-                    yield line[len(self.ps2) :]
-                elif line.rstrip():
-                    yield "# OUT: %s" % (line,)
+        if hasattr(self, 'all_logical_lines'):
+            # Curtsies
 
-        return "\n".join(process())
+            def process():
+                for line, lineType in self.all_logical_lines:
+                    if lineType == LineTypeTranslator.INPUT:
+                        yield line
+                    elif line.rstrip():
+                        yield "# OUT: %s" % line
+                yield "### %s" % self.current_line
+
+            return "\n".join(process())
+
+        else:  # cli and Urwid
+            session_output = self.getstdout()
+
+            def process():
+                for line in session_output.split("\n"):
+                    if line.startswith(self.ps1):	
+                        yield line[len(self.ps1) :]	
+                    elif line.startswith(self.ps2):	
+                        yield line[len(self.ps2) :]	
+                    elif line.rstrip():	        
+                        yield "# OUT: %s" % (line,)
+            return '\n'.join(process())
 
     def write2file(self):
         """Prompt for a filename and write the current contents of the stdout
@@ -867,7 +895,7 @@ class Repl(object):
                 self.interact.notify(_("Save cancelled."))
                 return
 
-        stdout_text = self.formatforfile(self.getstdout())
+        stdout_text = self.get_session_formatted_for_file()
 
         try:
             with open(fn, mode) as f:
@@ -884,7 +912,7 @@ class Repl(object):
             self.interact.notify(_("No clipboard available."))
             return
 
-        content = self.formatforfile(self.getstdout())
+        content = self.get_session_formatted_for_file()
         try:
             self.clipboard.copy(content)
         except CopyFailed:
@@ -940,6 +968,7 @@ class Repl(object):
     def push(self, s, insert_into_history=True):
         """Push a line of code onto the buffer so it can process it all
         at once when a code block ends"""
+        # This push method is used by cli and urwid, but not curtsies
         s = s.rstrip("\n")
         self.buffer.append(s)
 
@@ -1009,6 +1038,10 @@ class Repl(object):
 
         entries = list(self.rl_history.entries)
 
+        # Most recently undone command
+        last_entries = self.history[-n:]
+        last_entries.reverse()
+        self.redo_stack += last_entries
         self.history = self.history[:-n]
         self.reevaluate()
 
