@@ -102,7 +102,7 @@ class TestCurtsiesPaintingSimple(CurtsiesPaintingTest):
             orig_stdout = sys.stdout
             sys.stdout = self.repl.stdout
             [self.repl.add_normal_character(c) for c in "1 + 1"]
-            self.repl.on_enter(insert_into_history=False)
+            self.repl.on_enter(new_code=False)
             screen = fsarray([">>> 1 + 1", "2", "Welcome to"])
             self.assert_paint_ignoring_formatting(screen, (1, 1))
         finally:
@@ -248,7 +248,7 @@ class HigherLevelCurtsiesPaintingTest(CurtsiesPaintingTest):
             self.repl._set_cursor_offset(len(line), update_completion=False)
             self.repl.current_line = line
         with output_to_repl(self.repl):
-            self.repl.on_enter(insert_into_history=False)
+            self.repl.on_enter(new_code=False)
             self.assertEqual(self.repl.rl_history.entries, [""])
             self.send_refreshes()
 
@@ -272,6 +272,31 @@ class HigherLevelCurtsiesPaintingTest(CurtsiesPaintingTest):
         self.repl.paint()  # has some side effects we need to be wary of
 
 
+class TestWidthAwareness(HigherLevelCurtsiesPaintingTest):
+    def test_cursor_position_with_fullwidth_char(self):
+        self.repl.add_normal_character("間")
+
+        cursor_pos = self.repl.paint()[1]
+        self.assertEqual(cursor_pos, (0, 6))
+
+    def test_cursor_position_with_padding_char(self):
+        # odd numbered so fullwidth chars don't wrap evenly
+        self.repl.width = 11
+        [self.repl.add_normal_character(c) for c in "ｗｉｄｔｈ"]
+
+        cursor_pos = self.repl.paint()[1]
+        self.assertEqual(cursor_pos, (1, 4))
+
+    def test_display_of_padding_chars(self):
+        self.repl.width = 11
+        [self.repl.add_normal_character(c) for c in "ｗｉｄｔｈ"]
+
+        self.enter()
+        expected = [">>> ｗｉｄ ", "ｔｈ"]  # <--- note the added trailing space
+        result = [d.s for d in self.repl.display_lines[0:2]]
+        self.assertEqual(result, expected)
+
+
 class TestCurtsiesRewindRedraw(HigherLevelCurtsiesPaintingTest):
     def test_rewind(self):
         self.repl.current_line = "1 + 1"
@@ -592,7 +617,7 @@ class TestCurtsiesRewindRedraw(HigherLevelCurtsiesPaintingTest):
         self.repl.width = 50
         self.repl.current_line = "__import__('random').__name__"
         with output_to_repl(self.repl):
-            self.repl.on_enter(insert_into_history=False)
+            self.repl.on_enter(new_code=False)
         screen = [">>> __import__('random').__name__", "'random'"]
         self.assert_paint_ignoring_formatting(screen)
 
